@@ -2,29 +2,25 @@ use proc_macro2::{Span, TokenStream};
 use quote::{quote, quote_spanned};
 use syn::spanned::Spanned;
 
-use crate::internals::attr::{self, FieldAttr, Packing, Tag, TypeAttr};
+use crate::internals::attr::{self, DefaultTag, FieldAttr, Packing, TypeAttr};
 use crate::internals::symbol::*;
 use crate::internals::{Ctxt, Needs, NeedsKind};
 
 struct Tokens {
     decode_t: TokenStream,
     decoder_t: TokenStream,
+    decoder_var: syn::Ident,
     default_t: TokenStream,
     encode_t: TokenStream,
     encoder_t: TokenStream,
-    error_t: TokenStream,
-    pack_encoder_t: TokenStream,
-    struct_decoder_t: TokenStream,
-    struct_encoder_t: TokenStream,
-    struct_field_decoder_t: TokenStream,
-    tuple_decoder_t: TokenStream,
-    tuple_encoder_t: TokenStream,
-    tuple_field_decoder_t: TokenStream,
-    pack_decoder_t: TokenStream,
-    variant_decoder_t: TokenStream,
-    variant_encoder_t: TokenStream,
-    decoder_var: syn::Ident,
     encoder_var: syn::Ident,
+    error_t: TokenStream,
+    pack_decoder_t: TokenStream,
+    pack_encoder_t: TokenStream,
+    pair_decoder_t: TokenStream,
+    pair_encoder_t: TokenStream,
+    struct_decoder_t: TokenStream,
+    variant_encoder_t: TokenStream,
 }
 
 pub(crate) struct Expander<'a> {
@@ -49,22 +45,18 @@ impl<'a> Expander<'a> {
             tokens: Tokens {
                 decode_t: quote!(#prefix::de::Decode),
                 decoder_t: quote!(#prefix::de::Decoder),
+                decoder_var: syn::Ident::new("decoder", input.ident.span()),
                 default_t: quote!(::core::default::Default::default()),
                 encode_t: quote!(#prefix::en::Encode),
                 encoder_t: quote!(#prefix::en::Encoder),
-                error_t: quote!(#prefix::error::Error),
-                pack_encoder_t: quote!(#prefix::en::PackEncoder),
-                struct_decoder_t: quote!(#prefix::de::StructDecoder),
-                struct_encoder_t: quote!(#prefix::en::StructEncoder),
-                struct_field_decoder_t: quote!(#prefix::de::StructFieldDecoder),
-                tuple_decoder_t: quote!(#prefix::de::TupleDecoder),
-                tuple_encoder_t: quote!(#prefix::en::TupleEncoder),
-                tuple_field_decoder_t: quote!(#prefix::de::TupleFieldDecoder),
-                pack_decoder_t: quote!(#prefix::de::PackDecoder),
-                variant_decoder_t: quote!(#prefix::de::VariantDecoder),
-                variant_encoder_t: quote!(#prefix::en::VariantEncoder),
-                decoder_var: syn::Ident::new("decoder", input.ident.span()),
                 encoder_var: syn::Ident::new("encoder", input.ident.span()),
+                error_t: quote!(#prefix::error::Error),
+                pack_decoder_t: quote!(#prefix::de::PackDecoder),
+                pack_encoder_t: quote!(#prefix::en::PackEncoder),
+                pair_decoder_t: quote!(#prefix::de::PairDecoder),
+                pair_encoder_t: quote!(#prefix::en::PairEncoder),
+                struct_decoder_t: quote!(#prefix::de::StructDecoder),
+                variant_encoder_t: quote!(#prefix::en::VariantEncoder),
             },
         }
     }
@@ -278,28 +270,33 @@ impl<'a> Expander<'a> {
         let encoder_var = &self.tokens.encoder_var;
         let packing = self.type_attr.packing();
 
-        if let Some((field_kind, fields)) = FieldKind::new(fields) {
-            for (index, field) in fields.enumerate() {
-                needs.mark_used();
+        for (index, field) in fields.iter().enumerate() {
+            needs.mark_used();
 
-                let field_attr = attr::field_attrs(&self.cx, &field.attrs);
+            let field_attr = attr::field_attrs(&self.cx, &field.attrs);
 
-                let access = match &field.ident {
-                    Some(ident) => quote!(&self.#ident),
-                    None => {
-                        let n = field_int(index, field.span());
-                        quote!(&self.#n)
-                    }
-                };
-
-                let (encoder, skip) =
-                    self.encode_field(index, field, &field_attr, &access, field_kind, packing)?;
-                encoders.push(encoder);
-
-                if let Some((decl, test)) = skip {
-                    decls.push(decl);
-                    field_tests.push(test);
+            let access = match &field.ident {
+                Some(ident) => quote!(&self.#ident),
+                None => {
+                    let n = field_int(index, field.span());
+                    quote!(&self.#n)
                 }
+            };
+
+            let (encoder, skip) = self.encode_field(
+                index,
+                field,
+                &field_attr,
+                &access,
+                packing,
+                self.type_attr.default_field_tag,
+            )?;
+
+            encoders.push(encoder);
+
+            if let Some((decl, test)) = skip {
+                decls.push(decl);
+                field_tests.push(test);
             }
         }
 
@@ -460,7 +457,14 @@ impl<'a> Expander<'a> {
         let body = match self.type_attr.packing() {
             Packing::Tagged => {
                 needs.mark_used();
-                self.decode_tagged(&self.type_name, tag_type, path, &data.fields)?
+                self.decode_tagged(
+                    &self.type_name,
+                    tag_type,
+                    path,
+                    &data.fields,
+                    None,
+                    self.type_attr.default_field_tag,
+                )?
             }
             Packing::Packed => self.decode_untagged(path, &data.fields, needs)?,
             Packing::Transparent => self.decode_transparent(span, path, &data.fields, needs)?,
@@ -479,8 +483,9 @@ impl<'a> Expander<'a> {
         let error_t = &self.tokens.error_t;
         let type_ident = &self.input.ident;
         let type_name = &self.type_name;
-        let variant_decoder_t = &self.tokens.variant_decoder_t;
+        let pair_decoder_t = &self.tokens.pair_decoder_t;
         let decode_t = &self.tokens.decode_t;
+        let variant_tag = syn::Ident::new("variant_tag", self.input.ident.span());
 
         if let Some((span, Packing::Packed)) = self.type_attr.packing {
             self.cx.error_span(
@@ -505,10 +510,40 @@ impl<'a> Expander<'a> {
 
         let type_packing = self.type_attr.packing();
 
-        for (variant_index, variant) in data.variants.iter().enumerate() {
+        let mut fallback = None;
+        // Keep track of variant index manually since fallback variants do not
+        // count.
+        let mut variant_index = 0;
+
+        for variant in data.variants.iter() {
             let span = variant.span();
 
             let variant_attr = attr::variant_attrs(&self.cx, &variant.attrs);
+
+            if variant_attr.default.is_some() {
+                if !variant.fields.is_empty() {
+                    self.cx.error_span(
+                        variant.fields.span(),
+                        format!("#[{}({})] variant must be empty", ATTR, DEFAULT),
+                    );
+                    continue;
+                }
+
+                if fallback.is_some() {
+                    self.cx.error_span(
+                        variant.ident.span(),
+                        format!(
+                            "#[{}({})] only one fallback variant is supported",
+                            ATTR, DEFAULT
+                        ),
+                    );
+                    continue;
+                }
+
+                fallback = Some(&variant.ident);
+                continue;
+            }
+
             let variant_name = syn::LitStr::new(&variant.ident.to_string(), variant.ident.span());
 
             let mut path = syn::Path::from(syn::Ident::new("Self", type_ident.span()));
@@ -516,10 +551,19 @@ impl<'a> Expander<'a> {
 
             let tag_type = variant_attr.tag_type.as_ref();
 
+            let default_field_tag = variant_attr
+                .default_field_tag
+                .unwrap_or(self.type_attr.default_field_tag);
+
             let output = match variant_attr.packing().unwrap_or(type_packing) {
-                Packing::Tagged => {
-                    self.decode_tagged(&variant_name, tag_type, path, &variant.fields)?
-                }
+                Packing::Tagged => self.decode_tagged(
+                    &variant_name,
+                    tag_type,
+                    path,
+                    &variant.fields,
+                    Some(&variant_tag),
+                    default_field_tag,
+                )?,
                 Packing::Packed => self.decode_untagged(path, &variant.fields, needs)?,
                 Packing::Transparent => {
                     self.decode_transparent(span, path, &variant.fields, needs)?
@@ -528,16 +572,19 @@ impl<'a> Expander<'a> {
 
             let tag = self.expand_variant_tag(
                 variant_attr.rename.as_ref(),
-                self.type_attr.variant_tag,
+                self.type_attr.default_variant_tag,
                 variant_index,
                 &variant.ident,
             );
 
             variants.push(quote! {
                 #tag => {
+                    let #decoder_var = #pair_decoder_t::decode_second(variant)?;
                     #output
                 }
             });
+
+            variant_index += 1;
         }
 
         let tag_type = self
@@ -546,16 +593,33 @@ impl<'a> Expander<'a> {
             .as_ref()
             .map(|(_, ty)| quote!(: #ty));
 
+        let fallback = match fallback {
+            Some(ident) => {
+                quote! {
+                    if !#pair_decoder_t::skip_second(variant)? {
+                        return Err(<D::Error as #error_t>::unsupported_variant(#type_name, tag));
+                    }
+
+                    Self::#ident {}
+                }
+            }
+            None => quote! {
+                return Err(<D::Error as #error_t>::unsupported_variant(#type_name, tag));
+            },
+        };
+
         Some(quote! {{
             let mut variant = #decoder_t::decode_variant(#decoder_var)?;
-            let tag_decoder = #variant_decoder_t::decode_variant_tag(&mut variant)?;
-            let tag #tag_type = #decode_t::decode(tag_decoder)?;
-            let #decoder_var = #variant_decoder_t::decode_variant_value(variant)?;
 
-            Ok(match tag {
+            let #variant_tag #tag_type = {
+                let tag_decoder = #pair_decoder_t::decode_first(&mut variant)?;
+                #decode_t::decode(tag_decoder)?
+            };
+
+            Ok(match variant_tag {
                 #(#variants,)*
                 tag => {
-                    return Err(<D::Error as #error_t>::unsupported_variant(#type_name, tag));
+                    #fallback
                 }
             })
         }})
@@ -577,10 +641,10 @@ impl<'a> Expander<'a> {
                     #encoder_t::encode_struct(#encoder_var, #fields)?
                 };
 
-                let struct_encoder_t = &self.tokens.struct_encoder_t;
+                let pair_encoder_t = &self.tokens.pair_encoder_t;
 
                 let finish = Some(quote! {
-                    #struct_encoder_t::finish(#encoder_var)?;
+                    #pair_encoder_t::finish(#encoder_var)?;
                 });
 
                 Some((setup, finish))
@@ -592,10 +656,10 @@ impl<'a> Expander<'a> {
                     #encoder_t::encode_tuple(#encoder_var, #fields)?
                 };
 
-                let tuple_encoder_t = &self.tokens.tuple_encoder_t;
+                let pair_encoder_t = &self.tokens.pair_encoder_t;
 
                 let finish = Some(quote! {
-                    #tuple_encoder_t::finish(#encoder_var)?;
+                    #pair_encoder_t::finish(#encoder_var)?;
                 });
 
                 Some((setup, finish))
@@ -625,10 +689,14 @@ impl<'a> Expander<'a> {
             .packing()
             .unwrap_or_else(|| self.type_attr.packing());
 
+        let default_field_tag = variant_attr
+            .default_field_tag
+            .unwrap_or(self.type_attr.default_field_tag);
+
         let (mut encode, patterns) = match packing {
             Packing::Tagged => {
                 let (encode, patterns, tests) =
-                    self.encode_variant_fields(&variant.fields, needs, packing)?;
+                    self.encode_variant_fields(&variant.fields, needs, packing, default_field_tag)?;
 
                 // Special stuff needed to encode the field if its tagged.
                 let (setup, finish) = self.encode_field_tag(&variant.fields, &tests)?;
@@ -643,7 +711,7 @@ impl<'a> Expander<'a> {
             }
             Packing::Packed => {
                 let (encode, patterns, _) =
-                    self.encode_variant_fields(&variant.fields, needs, packing)?;
+                    self.encode_variant_fields(&variant.fields, needs, packing, default_field_tag)?;
                 (encode, patterns)
             }
             Packing::Transparent => self.transparent_variant(span, &variant.fields, needs)?,
@@ -661,7 +729,7 @@ impl<'a> Expander<'a> {
 
             let tag = self.expand_variant_tag(
                 variant_attr.rename.as_ref(),
-                self.type_attr.variant_tag,
+                self.type_attr.default_variant_tag,
                 variant_index,
                 &variant.ident,
             );
@@ -688,39 +756,44 @@ impl<'a> Expander<'a> {
         fields: &syn::Fields,
         needs: &mut Needs,
         tagged: Packing,
+        default_field_tag: DefaultTag,
     ) -> Option<(TokenStream, Vec<TokenStream>, Vec<syn::Ident>)> {
         let mut decls = Vec::with_capacity(fields.len());
         let mut encoders = Vec::with_capacity(fields.len());
         let mut patterns = Vec::with_capacity(fields.len());
         let mut field_tests = Vec::with_capacity(fields.len());
 
-        if let Some((field_kind, fields)) = FieldKind::new(fields) {
-            for (index, field) in fields.enumerate() {
-                needs.mark_used();
+        for (index, field) in fields.iter().enumerate() {
+            needs.mark_used();
 
-                let field_attr = attr::field_attrs(&self.cx, &field.attrs);
+            let field_attr = attr::field_attrs(&self.cx, &field.attrs);
 
-                let access = match &field.ident {
-                    Some(ident) => {
-                        patterns.push(quote!(#ident));
-                        quote!(#ident)
-                    }
-                    None => {
-                        let index = field_int(index, field.span());
-                        let var = syn::Ident::new(&format!("v{}", index), field.span());
-                        patterns.push(quote!(#index: #var));
-                        quote!(#var)
-                    }
-                };
-
-                let (encoder, skip) =
-                    self.encode_field(index, field, &field_attr, &access, field_kind, tagged)?;
-                encoders.push(encoder);
-
-                if let Some((decl, test)) = skip {
-                    decls.push(decl);
-                    field_tests.push(test);
+            let access = match &field.ident {
+                Some(ident) => {
+                    patterns.push(quote!(#ident));
+                    quote!(#ident)
                 }
+                None => {
+                    let index = field_int(index, field.span());
+                    let var = syn::Ident::new(&format!("v{}", index), field.span());
+                    patterns.push(quote!(#index: #var));
+                    quote!(#var)
+                }
+            };
+
+            let (encoder, skip) = self.encode_field(
+                index,
+                field,
+                &field_attr,
+                &access,
+                tagged,
+                default_field_tag,
+            )?;
+            encoders.push(encoder);
+
+            if let Some((decl, test)) = skip {
+                decls.push(decl);
+                field_tests.push(test);
             }
         }
 
@@ -756,8 +829,8 @@ impl<'a> Expander<'a> {
         field: &syn::Field,
         field_attr: &FieldAttr,
         access: &TokenStream,
-        field_kind: FieldKind,
         tagged: Packing,
+        default_field_tag: DefaultTag,
     ) -> Option<(TokenStream, Option<(TokenStream, syn::Ident)>)> {
         let encoder_var = &self.tokens.encoder_var;
         let encode_t = &self.tokens.encode_t;
@@ -768,35 +841,19 @@ impl<'a> Expander<'a> {
             Packing::Tagged | Packing::Transparent => {
                 let tag = self.expand_field_tag(
                     field_attr.rename.as_ref(),
-                    self.type_attr.field_tag,
+                    default_field_tag,
                     index,
                     field,
                 )?;
 
-                match field_kind {
-                    FieldKind::Struct => {
-                        let struct_encoder_t = &self.tokens.struct_encoder_t;
+                let pair_encoder_t = &self.tokens.pair_encoder_t;
 
-                        quote_spanned! {
-                            span => {
-                                let field_encoder = #struct_encoder_t::encode_field_tag(&mut #encoder_var)?;
-                                #encode_t::encode(&#tag, field_encoder)?;
-                                let value_encoder = #struct_encoder_t::encode_field_value(&mut #encoder_var)?;
-                                #encode_path(#access, value_encoder)?;
-                            }
-                        }
-                    }
-                    FieldKind::Tuple => {
-                        let tuple_encoder_t = &self.tokens.tuple_encoder_t;
-
-                        quote_spanned! {
-                            span => {
-                                let tag_encoder = #tuple_encoder_t::encode_field_tag(&mut #encoder_var)?;
-                                #encode_t::encode(&#tag, tag_encoder)?;
-                                let value_encoder = #tuple_encoder_t::encode_field_value(&mut #encoder_var)?;
-                                #encode_path(#access, value_encoder)?;
-                            }
-                        }
+                quote_spanned! {
+                    span => {
+                        let field_encoder = #pair_encoder_t::encode_first(&mut #encoder_var)?;
+                        #encode_t::encode(&#tag, field_encoder)?;
+                        let value_encoder = #pair_encoder_t::encode_second(&mut #encoder_var)?;
+                        #encode_path(#access, value_encoder)?;
                     }
                 }
             }
@@ -838,6 +895,8 @@ impl<'a> Expander<'a> {
         tag_type: Option<&(Span, syn::Type)>,
         path: syn::Path,
         fields: &syn::Fields,
+        variant_tag: Option<&syn::Ident>,
+        default_field_tag: DefaultTag,
     ) -> Option<TokenStream> {
         let decode_t = &self.tokens.decode_t;
         let decoder_t = &self.tokens.decoder_t;
@@ -863,38 +922,19 @@ impl<'a> Expander<'a> {
         for (index, field) in fields.enumerate() {
             let field_attr = attr::field_attrs(&self.cx, &field.attrs);
 
-            let tag = self.expand_field_tag(
-                field_attr.rename.as_ref(),
-                self.type_attr.field_tag,
-                index,
-                field,
-            )?;
+            let tag =
+                self.expand_field_tag(field_attr.rename.as_ref(), default_field_tag, index, field)?;
 
             let (span, decode_path) = field_attr.decode_path(decode_t, field.span());
             let var = syn::Ident::new(&format!("v{}", index), span);
             decls.push(quote_spanned!(span => let mut #var = None;));
             let decode = quote_spanned!(span => #var = Some(#decode_path(#decoder_var)?));
 
-            let prefix = match field_kind {
-                FieldKind::Struct => {
-                    let struct_field_decoder_t = &self.tokens.struct_field_decoder_t;
-
-                    Some(quote! {
-                        let #decoder_var = #struct_field_decoder_t::decode_field_value(&mut #decoder_var)?;
-                    })
-                }
-                FieldKind::Tuple => {
-                    let tuple_field_decoder_t = &self.tokens.tuple_field_decoder_t;
-
-                    Some(quote! {
-                        let #decoder_var = #tuple_field_decoder_t::decode_field_value(&mut #decoder_var)?;
-                    })
-                }
-            };
+            let pair_decoder_t = &self.tokens.pair_decoder_t;
 
             patterns.push(quote! {
                 #tag => {
-                    #prefix
+                    let #decoder_var = #pair_decoder_t::decode_second(#decoder_var)?;
                     #decode;
                 }
             });
@@ -910,7 +950,7 @@ impl<'a> Expander<'a> {
             let fallback = if let Some(span) = field_attr.default {
                 quote_spanned!(span => #default_t)
             } else {
-                quote!(return Err(<D::Error as #error_t>::missing_field(#type_name, #tag)))
+                quote!(return Err(<D::Error as #error_t>::expected_tag(#type_name, #tag)))
             };
 
             assigns.push(quote!(#field: match #var {
@@ -934,84 +974,65 @@ impl<'a> Expander<'a> {
                 quote!(#struct_decoder_t::decode_field)
             }
             FieldKind::Tuple => {
-                let tuple_decoder_t = &self.tokens.tuple_decoder_t;
-                quote!(#tuple_decoder_t::decode_field)
+                let struct_decoder_t = &self.tokens.struct_decoder_t;
+                quote!(#struct_decoder_t::decode_field)
             }
         };
 
-        let (decode_tag, skip_field) = match field_kind {
-            FieldKind::Struct => {
-                let struct_field_decoder_t = &self.tokens.struct_field_decoder_t;
-                let decode_t = &self.tokens.decode_t;
+        let pair_decoder_t = &self.tokens.pair_decoder_t;
+        let decode_t = &self.tokens.decode_t;
 
-                let decode_tag = quote! {{
-                    let name_decoder = #struct_field_decoder_t::decode_field_tag(&mut #decoder_var)?;
-                    #decode_t::decode(name_decoder)?
-                }};
+        let decode_tag = quote! {{
+            let index_decoder = #pair_decoder_t::decode_first(&mut #decoder_var)?;
+            #decode_t::decode(index_decoder)?
+        }};
 
-                let skip_field = quote! {
-                    #struct_field_decoder_t::skip_field_value(&mut #decoder_var)?
-                };
+        let skip_field = quote! {
+            #pair_decoder_t::skip_second(#decoder_var)?
+        };
 
-                (decode_tag, skip_field)
-            }
-            FieldKind::Tuple => {
-                let tuple_field_decoder_t = &self.tokens.tuple_field_decoder_t;
-                let decode_t = &self.tokens.decode_t;
-
-                let decode_tag = quote! {{
-                    let index_decoder = #tuple_field_decoder_t::decode_field_tag(&mut #decoder_var)?;
-                    #decode_t::decode(index_decoder)?
-                }};
-
-                let skip_field = quote! {
-                    #tuple_field_decoder_t::skip_field_value(&mut #decoder_var)?
-                };
-
-                (decode_tag, skip_field)
-            }
+        let unsupported = match variant_tag {
+            Some(variant_tag) => quote! {
+                <D::Error as #error_t>::unsupported_variant_field(#type_name, #variant_tag, tag)
+            },
+            None => quote! {
+                <D::Error as #error_t>::unsupported_field(#type_name, tag)
+            },
         };
 
         let tag_type = tag_type.as_ref().map(|(_, ty)| quote!(: #ty));
 
-        if patterns.is_empty() {
-            return Some(quote! {
-                let mut #decoder_var = #declare;
-
-                while let Some(mut #decoder_var) = #decode_next(&mut #decoder_var)? {
-                    let tag #tag_type = #decode_tag;
-
-                    if !#skip_field {
-                        return Err(<D::Error as #error_t>::unsupported_tag(#type_name, tag));
-                    }
+        let body = if patterns.is_empty() {
+            quote! {
+                if !#skip_field {
+                    return Err(#unsupported);
                 }
+            }
+        } else {
+            quote! {
+                match tag {
+                    #(#patterns,)*
+                    tag => {
+                        if !#skip_field {
+                            return Err(#unsupported);
+                        }
+                    },
+                }
+            }
+        };
 
-                #path {}
-            });
-        }
-
-        return Some(quote! {
+        Some(quote! {
             #(#decls;)*
 
             let mut #decoder_var = #declare;
 
             while let Some(mut #decoder_var) = #decode_next(&mut #decoder_var)? {
                 let tag #tag_type = #decode_tag;
-
-                match tag {
-                    #(#patterns,)*
-                    tag => {
-                        if !#skip_field {
-                            return Err(<D::Error as #error_t>::unsupported_tag(#type_name, tag));
-                        }
-                    },
-                }
+                #body
             }
 
-            #path {
-                #(#assigns),*
-            }
-        });
+            #path { #(#assigns),* }
+        })
     }
 
     /// Decode something packed.
@@ -1079,14 +1100,14 @@ impl<'a> Expander<'a> {
     fn expand_variant_tag(
         &self,
         rename: Option<&(Span, syn::Expr)>,
-        tag: Tag,
+        tag: DefaultTag,
         index: usize,
         ident: &syn::Ident,
     ) -> Option<syn::Expr> {
         let lit = match (rename, tag) {
             (Some((_, value)), _) => return Some(self.rename_lit(value)),
-            (None, Tag::Index) => usize_int(index, ident.span()).into(),
-            (None, Tag::Name) => syn::LitStr::new(&ident.to_string(), ident.span()).into(),
+            (None, DefaultTag::Index) => usize_int(index, ident.span()).into(),
+            (None, DefaultTag::Name) => syn::LitStr::new(&ident.to_string(), ident.span()).into(),
         };
 
         Some(syn::Expr::Lit(syn::ExprLit {
@@ -1099,14 +1120,14 @@ impl<'a> Expander<'a> {
     fn expand_field_tag(
         &self,
         rename: Option<&(Span, syn::Expr)>,
-        tag: Tag,
+        default_field_tag: DefaultTag,
         index: usize,
         field: &syn::Field,
     ) -> Option<syn::Expr> {
-        let lit = match (rename, tag, &field.ident) {
+        let lit = match (rename, default_field_tag, &field.ident) {
             (Some((_, rename)), _, _) => return Some(self.rename_lit(rename)),
-            (None, Tag::Index, _) => usize_int(index, field.span()).into(),
-            (None, Tag::Name, None) => {
+            (None, DefaultTag::Index, _) => usize_int(index, field.span()).into(),
+            (None, DefaultTag::Name, None) => {
                 self.cx.error_spanned_by(
                     field,
                     format!(
@@ -1116,7 +1137,7 @@ impl<'a> Expander<'a> {
                 );
                 return None;
             }
-            (None, Tag::Name, Some(ident)) => {
+            (None, DefaultTag::Name, Some(ident)) => {
                 syn::LitStr::new(&ident.to_string(), ident.span()).into()
             }
         };
